aboutsummaryrefslogtreecommitdiff
path: root/gcc
diff options
context:
space:
mode:
Diffstat (limited to 'gcc')
-rw-r--r--gcc/cp/pt.c6
-rw-r--r--gcc/testsuite/g++.dg/cpp2a/concepts-requires23.C9
2 files changed, 13 insertions, 2 deletions
diff --git a/gcc/testsuite/g++.dg/cpp2a/concepts-requires23.C b/gcc/testsuite/g++.dg/cpp2a/concepts-requires23.C
new file mode 100644
index 0000000..e109bea
--- /dev/null
+++ b/gcc/testsuite/g++.dg/cpp2a/concepts-requires23.C
@@ -0,0 +1,9 @@
+// { dg-do compile { target c++20 } }
+
+// Verify f<A>'s associated constraints evaluate to false due
+// to return type deduction failure for A::foo().
+
+template <class T> concept fooable = requires { T::foo(0); };
+template <fooable T> int f ();
+struct A { static auto *foo(auto); };
+int a = f<A>(); // { dg-error "no match" }
diff --git a/gcc/cp/pt.c b/gcc/cp/pt.c
index e5f18d2..5100954 100644
--- a/gcc/cp/pt.c
+++ b/gcc/cp/pt.c
@@ -10805,14 +10805,16 @@ uses_template_parms (tree t)
return dependent_p;
}
/* Returns true iff current_function_decl is an incompletely instantiated
template. Useful instead of processing_template_decl because the latter
is set to 0 during instantiate_non_dependent_expr. */
bool
in_template_function (void)
{
- tree fn = current_function_decl;
+ /* Inspect the less volatile cfun->decl instead of current_function_decl;
+ the latter might get set for e.g. access checking during satisfaction. */
+ tree fn = cfun ? cfun->decl : NULL_TREE;
bool ret;
++processing_template_decl;
ret = (fn && DECL_LANG_SPECIFIC (fn)
@@ -10805,14 +10805,16 @@ uses_template_parms (tree t)
return dependent_p;
}
-/* Returns true iff current_function_decl is an incompletely instantiated
+/* Returns true iff we're processing an incompletely instantiated function
template. Useful instead of processing_template_decl because the latter
is set to 0 during instantiate_non_dependent_expr. */